// Copyright Joyent, Inc. and other Node contributors.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.

var common = require('../common');
var assert = require('assert');
var path = require('path');
var fs = require('fs');
var got_error = false;
var success_count = 0;

var file = path.join(common.fixturesDir, 'a.js');

fs.chmod(file, '0777', function(err) {
  if (err) {
    got_error = true;
  } else {
    console.log(fs.statSync(file).mode);
    assert.equal(0777, fs.statSync(file).mode & 0777);

    fs.chmodSync(file, 0644);
    assert.equal(0644, fs.statSync(file).mode & 0777);
    success_count++;
  }
});

fs.open(file, 'a', function(err, fd) {
  if (err) {
    got_error = true;
    console.error(err.stack);
    return;
  }
  fs.fchmod(fd, '0777', function(err) {
    if (err) {
      got_error = true;
    } else {
      console.log(fs.fstatSync(fd).mode);
      assert.equal(0777, fs.fstatSync(fd).mode & 0777);

      fs.fchmodSync(fd, 0644);
      assert.equal(0644, fs.fstatSync(fd).mode & 0777);
      success_count++;
    }
  });
});

process.addListener('exit', function() {
  assert.equal(2, success_count);
  assert.equal(false, got_error);
});

